@@ -22,13 +22,9 @@
 
 #include "sourcesmodel.h"
 
-// FIXME: At some point these might need to become plugins
-#include "sources/baloosource.h"
-#include "sources/plasmarunnersource.h"
-#include "sources/applicationsource.h"
-#include "sources/calculatorsource.h"
-
-#include <KDebug>
+#include <QDebug>
+#include <KConfig>
+#include <KConfigGroup>
 #include <QTimer>
 
 using namespace Milou;
@@ -37,91 +33,32 @@ SourcesModel::SourcesModel(QObject* parent)
     : QAbstractListModel(parent)
     , m_size(0)
 {
-    //PlasmaRunnerSource* plasmaRunnerSource = new PlasmaRunnerSource(this);
-    //connect(plasmaRunnerSource, SIGNAL(matchAdded(Match)), this, SLOT(slotMatchAdded(Match)));
-
-    ApplicationSource* appSource = new ApplicationSource(this);
-    connect(appSource, SIGNAL(matchAdded(Match)), this, SLOT(slotMatchAdded(Match)));
-
-    AbstractSource* balooSource = new BalooSource(this);
-    connect(balooSource, SIGNAL(matchAdded(Match)), this, SLOT(slotMatchAdded(Match)));
-
-    AbstractSource* calculatorsource = new CalculatorSource(this);
-    connect(calculatorsource, SIGNAL(matchAdded(Match)), this, SLOT(slotMatchAdded(Match)));
-
-    m_sources << appSource;
-    m_sources << balooSource;
-    //m_sources << plasmaRunnerSource;
-    m_sources << calculatorsource;
-
-    QHash<int, QByteArray> roles = roleNames();
-    roles.insert(TypeRole, "type");
-    roles.insert(PreviewTypeRole, "previewType");
-    roles.insert(PreviewUrlRole, "previewUrl");
-    roles.insert(PreviewLabelRole, "previewLabel");
-
-    setRoleNames(roles);
-    loadSettings();
+    m_manager = new Plasma::RunnerManager(this);
+    connect(m_manager, SIGNAL(matchesChanged(QList<Plasma::QueryMatch>)),
+            this, SLOT(slotMatchesChanged(QList<Plasma::QueryMatch>)));
 }
 
 SourcesModel::~SourcesModel()
 {
 }
 
-void SourcesModel::loadSettings()
+QHash<int, QByteArray> SourcesModel::roleNames() const
 {
-    QList<MatchType*> allTypes;
-    foreach(AbstractSource* source, m_sources)
-        allTypes << source->types();
+    QHash<int, QByteArray> roles = QAbstractListModel::roleNames();
+    roles.insert(TypeRole, "type");
+    roles.insert(PreviewTypeRole, "previewType");
+    roles.insert(PreviewUrlRole, "previewUrl");
+    roles.insert(PreviewLabelRole, "previewLabel");
 
-    KConfig config("milourc");
-    KConfigGroup generalGroup = config.group("General");
-    int numTypes = generalGroup.readEntry("NumTypes", 0);
-
-    if (numTypes != allTypes.size()) {
-        generalGroup.writeEntry("NumTypes", allTypes.size());
-
-        for(int i=0; i<allTypes.size(); i++) {
-            MatchType* type = allTypes[i];
-            KConfigGroup group = config.group("Type-" + QString::number(i));
-
-            group.writeEntry("Name", type->name());
-            group.writeEntry("Icon", type->icon());
-            group.writeEntry("Enabled", true);
-
-            m_types << type->name();
-        }
-    }
-    else {
-        kDebug() << "Loading the settings";
-        m_types.resize(allTypes.size());
-        m_typesShown.clear();
-
-        for(int i=0; i<allTypes.size(); i++) {
-            KConfigGroup group = config.group("Type-" + QString::number(i));
-
-            QString name = group.readEntry("Name", QString());
-            bool shown = group.readEntry("Enabled", true);
-
-            // Update allTypes
-            foreach(MatchType* type, allTypes) {
-                if (type->name() == name) {
-                    kDebug() << i << type->name() << shown;
-                    if (shown)
-                        m_typesShown << type;
-                    m_types[i] = name;
-                }
-            }
-        }
-    }
+    return roles;
 }
 
-Match SourcesModel::fetchMatch(int row) const
+Plasma::QueryMatch SourcesModel::fetchMatch(int row) const
 {
-    foreach(const QString& type, m_types) {
+    foreach (const QString& type, m_types) {
         const TypeData data = m_matches.value(type);
         if (row < data.shown.size()) {
-            return data.shown.value(row);
+            return data.shown[row];
         }
         else {
             row -= data.shown.size();
@@ -131,7 +68,7 @@ Match SourcesModel::fetchMatch(int row) const
         }
     }
 
-    return Match(0);
+    return Plasma::QueryMatch(0);
 }
 
 QVariant SourcesModel::data(const QModelIndex& index, int role) const
@@ -142,24 +79,20 @@ QVariant SourcesModel::data(const QModelIndex& index, int role) const
     if (index.row() >= m_size)
         return QVariant();
 
-    Match m = fetchMatch(index.row());
-    Q_ASSERT(m.source());
+    Plasma::QueryMatch m = fetchMatch(index.row());
+    Q_ASSERT(m.runner());
 
     switch(role) {
         case Qt::DisplayRole:
             return m.text();
 
-        case Qt::DecorationRole: {
-            QString icon = m.icon();
-            if (!icon.isEmpty())
-                return icon;
-            else
-                return m.type()->icon();
-        }
+        case Qt::DecorationRole:
+            return m.icon().name();
 
         case TypeRole:
-            return m.type()->name();
+            return m.matchCategory();
 
+            /*
         case PreviewTypeRole:
             return m.previewType();
 
@@ -168,6 +101,7 @@ QVariant SourcesModel::data(const QModelIndex& index, int role) const
 
         case PreviewLabelRole:
             return m.previewLabel();
+            */
     }
 
     return QVariant();
@@ -194,8 +128,10 @@ int SourcesModel::queryLimit() const
 void SourcesModel::setQueryLimit(int limit)
 {
     m_queryLimit = limit;
+    /*
     foreach (AbstractSource* source, m_sources)
         source->setQueryLimit(limit);
+    */
 }
 
 void SourcesModel::setQueryString(const QString& str)
@@ -209,15 +145,12 @@ void SourcesModel::setQueryString(const QString& str)
 
     m_queryString = str;
 
-    Context context;
-    context.setQuery(str);
-    context.setTypes(m_typesShown);
-
+    m_manager->reset();
+    m_types.clear();
+    m_typePriority.clear();
     m_supressSignals = true;
-    foreach (AbstractSource* source, m_sources) {
-        source->query(context);
-    }
 
+    m_manager->launchQuery(m_queryString);
     QTimer::singleShot(250, this, SLOT(stopSuppressingSignals()));
 }
 
@@ -233,16 +166,47 @@ void SourcesModel::stopSuppressingSignals()
 // Tries to make sure that all the types have the same number
 // of visible items
 //
-void SourcesModel::slotMatchAdded(const Match& m)
+void SourcesModel::slotMatchesChanged(const QList<Plasma::QueryMatch>& list)
+{
+    Q_FOREACH (const Plasma::QueryMatch& match, list) {
+        slotMatchAdded(match);
+    }
+}
+
+void SourcesModel::slotMatchAdded(const Plasma::QueryMatch& m)
 {
     if (m_queryString.isEmpty())
         return;
 
-    if (!m_typesShown.contains(m.type()))
-        return;
+    QString matchType = m.matchCategory();
 
-    const QString matchType = m.type()->name();
-    //Q_ASSERT(m.source()->types().contains(matchType));
+    if (!m_types.contains(matchType)) {
+        int priority = static_cast<int>(m.type());
+        if (m.runner()->name() == QStringLiteral("Applications"))
+            priority = 100000; // Really high number
+
+        m_typePriority[matchType] = priority;
+
+        QMutableListIterator<QString> it(m_types);
+        while (it.hasNext()) {
+            QString t = it.next();
+            int p = m_typePriority.value(t);
+            if (p < priority) {
+                it.previous();
+                it.insert(matchType);
+                break;
+            }
+        }
+
+        if (m_types.isEmpty() || !m_types.contains(matchType)) {
+            m_types << matchType;
+        }
+
+        if (!m_supressSignals) {
+            beginResetModel();
+            endResetModel();
+        }
+    }
 
     if (m_size == m_queryLimit) {
         int maxShownItems = 0;
@@ -267,7 +231,7 @@ void SourcesModel::slotMatchAdded(const Match& m)
 
         if (!m_supressSignals)
             beginRemoveRows(QModelIndex(), removeRowPos, removeRowPos);
-        Match transferMatch = m_matches[maxShownType].shown.takeLast();
+        Plasma::QueryMatch transferMatch = m_matches[maxShownType].shown.takeLast();
         m_matches[maxShownType].hidden.append(transferMatch);
         m_size--;
         if (!m_supressSignals)
@@ -318,16 +282,12 @@ void SourcesModel::clear()
     m_matches.clear();
     m_size = 0;
     m_queryString.clear();
-    foreach (AbstractSource* source, m_sources)
-        source->stop();
+    m_manager->reset();
     endResetModel();
 }
 
 void SourcesModel::run(int index)
 {
-    Match match = fetchMatch(index);
-    AbstractSource* source = match.source();
-    if (source) {
-        source->run(match);
-    }
+    Plasma::QueryMatch match = fetchMatch(index);
+    m_manager->run(match);
 }
