@@ -26,7 +26,9 @@
 #include <QAbstractItemModel>
 #include <QVector>
 
-#include "abstractsource.h"
+#include <KRunner/RunnerManager>
+#include <KRunner/QueryMatch>
+
 #include "milou_export.h"
 
 namespace Milou {
@@ -53,26 +55,36 @@ public:
     QString queryString() const;
     int queryLimit() const;
 
+    virtual QHash<int, QByteArray> roleNames() const;
+
 public slots:
     void setQueryString(const QString& str);
     void setQueryLimit(int limit);
     void clear();
 
     void run(int index);
-    void loadSettings();
+
+    Q_INVOKABLE QString getType(int index) const {
+        return data(createIndex(index, 0), TypeRole).toString();
+    }
+
 
 private slots:
-    void slotMatchAdded(const Match& m);
+    void slotMatchesChanged(const QList<Plasma::QueryMatch>& list);
+    void slotMatchAdded(const Plasma::QueryMatch& match);
     void stopSuppressingSignals();
 
 public:
-    // The types are ordered based on the preference
-    QVector<QString> m_types;
-    QList<MatchType*> m_typesShown;
+    // A list of all the types that are being shown
+    QList<QString> m_types;
+
+    // Each type has a priority based on the results, on how high it should
+    // be on the list
+    QHash<QString, int> m_typePriority;
 
     struct TypeData {
-        QList<Match> shown;
-        QList<Match> hidden;
+        QList<Plasma::QueryMatch> shown;
+        QList<Plasma::QueryMatch> hidden;
     };
     QHash<QString, TypeData> m_matches;
     int m_size;
@@ -80,13 +92,12 @@ public:
     QString m_queryString;
     int m_queryLimit;
 
-    QList<AbstractSource*> m_sources;
-
     bool m_supressSignals;
+    Plasma::RunnerManager* m_manager;
 
     /// Returns the number of visible rows before \p type
     int fetchRowCount(const QString& type) const;
-    Match fetchMatch(int row) const;
+    Plasma::QueryMatch fetchMatch(int row) const;
 };
 
 }
